<?php

namespace Bluesquare\Pilot;

class Pilot
{
    /**
     * @var callable[]
     */
    protected $registry = [
        'action' => [],
        'form' => [],
        'metric' => [],
    ];

    public function __construct()
    {

    }

    public function authorize($token, $user_token)
    {
        if ($token !== $user_token) {
            throw new \Exception('Unauthorized Pilot request');
        }
    }

    public function form($slug, callable $function)
    {
        $this->register('form', $slug, function () use ($function) {
            return [
                'json' => $function(),
            ];
        });
    }

    public function action($slug, callable $function)
    {
        $this->register('action', $slug, $function);
    }

    public function metric($slug, callable $function)
    {
        $this->register('metric', $slug, $function);
    }

    protected function register($type, $slug, callable $function)
    {
        $this->registry[$type][$slug] = $function;
    }

    public function handle($data = [], $files = [])
    {
        $type = $data['type'] ?? null;
        $slug = $data['slug'] ?? null;

        if (isset($this->registry[$type]) && isset($this->registry[$type][$slug])) {
            $params = $this->resolveDependencies($slug, $this->registry[$type][$slug], $data, $files);

            return call_user_func_array($this->registry[$type][$slug], $params);
        }

        return Response::error("Unknown {$type} {$slug}");
    }

    public function resolveDependencies($slug, $function, $data, $files)
    {
        $entries = isset($data['entries']) && is_array($data['entries']) ? $data['entries'] : [];

        $input = [
            'data' => $data,
            'filter' => $data['filter'] ?? null,
            'files' => $files,
            'entries' => $entries,
            'entry' => $entries[0] ?? null,
        ];

        $output = [];

        $ref = is_array($function) ? new \ReflectionMethod($function[0], $function[1]) : new \ReflectionFunction($function);

        foreach ($ref->getParameters() as $parameter) {
            if (isset($input[$parameter->getName()])) {
                $output[$parameter->getName()] = $input[$parameter->getName()];
            } elseif (! $parameter->isOptional()) {
                throw new \Exception("Pilot[$slug]: unknown required parameter '{$parameter->getName()}'");
            }
        }

        return $output;
    }

    public function serve($data)
    {
        if (isset($data['json'])) {
            header('content-type: application/json');
            echo json_encode($data['json']);
        } elseif (isset($data['file'])) {
            header('content-type: '.$data['file']['type']);

            if (! empty($data['file']['name'])) {
                header('content-disposition: attachment; filename="'.$data['file']['name'].'"');
            }

            echo $data['file']['content'];
        }
    }
}
